use serde::Serialize;
use sqlx::PgPool;

#[derive(Serialize)]
pub struct User {
    pub id: uuid::Uuid,
    pub email: String,
    pub admin: bool,
}

use crate::auth::AuthError;
use crate::database::model::User as DbUser;
use crate::database::model::UserMember as DbUserMember;
use crate::util::convert_vec;

use super::member::Roles;
use super::Member;
impl From<DbUser> for User {
    fn from(db_user: DbUser) -> Self {
        Self {
            id: db_user.user_id,
            email: db_user.email,
            admin: db_user.admin,
        }
    }
}

impl From<User> for DbUser {
    fn from(user: User) -> Self {
        Self {
            user_id: user.id,
            email: user.email,
            admin: user.admin,
            password: None,
        }
    }
}

impl User {
    pub async fn members(&self, pool: &PgPool) -> Result<Vec<Member>, sqlx::Error> {
        let related_members = DbUserMember::get_members_from_user(pool, &self.id).await?;

        Ok(convert_vec(related_members))
    }

    pub async fn authorize(
        &self,
        pool: &PgPool,
        required_roles: Option<Roles>,
        requested_user_id: Option<String>,
    ) -> Result<(), AuthError> {
        if let Some(user_id) = requested_user_id {
            let user_uuid = uuid::Uuid::parse_str(&user_id).map_err(|_| AuthError::NoPermssions)?;

            if self.id != user_uuid {
                return Err(AuthError::NoPermssions);
            }

            return Ok(());
        }

        if let Some(roles) = required_roles {
            let user_roles = DbUserMember::get_roles(pool, &self.id)
                .await
                .unwrap_or(Roles::MEMBER);
            if !user_roles.intersects(roles) {
                return Err(AuthError::NoPermssions);
            }

            return Ok(());
        }

        Ok(())
    }
}
