use axum::{
    extract::State,
    http::HeaderMap,
    routing::post,
    Json, Router,
};
use serde::Deserialize;

use crate::{
    auth::get_user_from_header,
    database::model::Message as DbMessage,
    model::{
        member::{Groups, Roles},
        message::{Channel, MessageCreate},
        Message,
    },
    AppState,
};

pub fn routes() -> Router<AppState> {
    Router::new().route("/messages", post(message_create))
}

#[derive(Debug, Deserialize)]
pub struct MessageCreateRequest {
    title: String,
    content: String,
    channel: String,
    member_groups: Option<String>,
    member_roles: Option<String>,
}

pub async fn message_create(
    State(state): State<AppState>,
    headers: HeaderMap,
    Json(request): Json<MessageCreateRequest>,
) -> Result<(), crate::Error> {
    let user = get_user_from_header(&state.pool, &headers).await?;

    user.authorize(&state.pool, Some(Roles::ADMIN | Roles::MESSAGES), None)
        .await?;

    let channel: Channel = bitflags::parser::from_str_strict(&request.channel).map_err(|_| {
        crate::Error::BadRequest {
            expected: String::from("Error while parsing channel"),
        }
    })?;

    let member_groups: Option<Groups> = request
        .member_groups
        .map(|groups| {
            bitflags::parser::from_str_strict(&groups).map_err(|_| crate::Error::BadRequest {
                expected: String::from("Error while parsing member groups"),
            })
        })
        .transpose()?;

    let member_roles: Option<Roles> = request
        .member_roles
        .map(|roles| {
            bitflags::parser::from_str_strict(&roles).map_err(|_| crate::Error::BadRequest {
                expected: String::from("Error while parsing member roles"),
            })
        })
        .transpose()?;

    let db_message: DbMessage = Message::new(MessageCreate {
        title: request.title,
        content: request.content,
        channel,
        member_groups,
        member_roles,
        scheduled_at: None,
        thumbnail_url: None,
    })
    .into();

    let mut transaction = state.pool.begin().await?;

    db_message.insert(&mut transaction).await?;

    transaction.commit().await?;

    Ok(())
}
