use axum::{
    http::StatusCode,
    response::{IntoResponse, Response},
    Json,
};

#[derive(thiserror::Error, Debug)]
pub enum Error {
    #[error(transparent)]
    Csv(#[from] csv::Error),

    #[error(transparent)]
    Auth(#[from] crate::auth::AuthError),

    #[error(transparent)]
    Sqlx(#[from] sqlx::Error),

    #[error("resource not found")]
    NotFound,

    #[error("Invalid request, expected {expected:?}")]
    BadRequest { expected: String },
}

#[derive(serde::Serialize)]
struct ErrorResponse {
    code: &'static str,
    description: String,
}

impl ErrorResponse {
    fn new(code: &'static str, description: String) -> Self {
        Self { code, description }
    }
}

impl IntoResponse for Error {
    fn into_response(self) -> Response {
        let (status_code, code) = match self {
            Self::Sqlx(ref err_kind) => match err_kind {
                sqlx::Error::RowNotFound => (StatusCode::NOT_FOUND, "DATABASE_ROW_NOT_FOUND"),
                sqlx::Error::Database(db_err) => match db_err.kind() {
                    sqlx::error::ErrorKind::UniqueViolation => {
                        (StatusCode::INTERNAL_SERVER_ERROR, "DATABASE_DUPLICATE")
                    }
                    _ => (StatusCode::INTERNAL_SERVER_ERROR, "DATABASE_ERROR"),
                },
                _ => (StatusCode::INTERNAL_SERVER_ERROR, "DATABASE_ERROR"),
            },

            Self::Csv(ref err_kind) => match err_kind.kind() {
                csv::ErrorKind::UnequalLengths { .. } => {
                    (StatusCode::BAD_REQUEST, "CSV_INPUT_INVALID")
                }
                _ => (StatusCode::INTERNAL_SERVER_ERROR, "CSV_ERROR"),
            },

            Self::Auth(ref err_kind) => match err_kind {
                crate::auth::AuthError::NoPermssions => {
                    (StatusCode::FORBIDDEN, "AUTH_NO_PERMISSIONS")
                }
                crate::auth::AuthError::Unauthorized => {
                    (StatusCode::UNAUTHORIZED, "AUTH_UNAUTHORIZED")
                }
                crate::auth::AuthError::InvalidToken => {
                    (StatusCode::BAD_REQUEST, "AUTH_INVALID_TOKEN")
                }
                crate::auth::AuthError::InvalidPassword => {
                    (StatusCode::BAD_REQUEST, "AUTH_INVALID_PASSWORD")
                }
                _ => (StatusCode::INTERNAL_SERVER_ERROR, "AUTH_ERROR"),
            },

            Self::BadRequest { .. } => (StatusCode::BAD_REQUEST, "INVALID_BODY"),
            Self::NotFound => (StatusCode::NOT_FOUND, "NOT_FOUND"),
        };

        let body = ErrorResponse::new(code, self.to_string());

        (status_code, Json(body)).into_response()
    }
}
